@@ -46,12 +46,24 @@ int exit_sync_needed = 2;
  */
 static BOOLEAN (WINAPI* ptr_RtlGenRandom)(PVOID, ULONG);
 
+/**
+ * Direct access to GetTickCount64().
+ * This function is available only from Windows Vista.
+ */
+static ULONGLONG (WINAPI* ptr_GetTickCount64)(void);
+
 /**
  * Description of the last error.
  * It's stored in the thread local storage.
  */
 static pthread_key_t last_error;
 
+/**
+ * Monotone tick counter
+ */
+static pthread_mutex_t tick_lock;
+static uint64_t tick_last;
+
 /**
  * If we are running in Wine.
  */
@@ -112,6 +124,12 @@ void os_init(int opt)
 		exit(EXIT_FAILURE);
 	}
 
+	tick_last = 0;
+	if (pthread_mutex_init(&tick_lock, 0) != 0) {
+		log_fatal("Error calling pthread_mutex_init().\n");
+		exit(EXIT_FAILURE);
+	}
+
 	ntdll = GetModuleHandle("NTDLL.DLL");
 	if (!ntdll) {
 		log_fatal("Error loading the NTDLL module.\n");
@@ -139,6 +157,9 @@ void os_init(int opt)
 	/* get pointer to RtlGenRandom, note that it was reported missing in some cases */
 	ptr_RtlGenRandom = (void*)GetProcAddress(dll_advapi32, "SystemFunction036");
 
+	/* get pointer to RtlGenRandom, note that it was reported missing in some cases */
+	ptr_GetTickCount64 = (void*)GetProcAddress(kernel32, "GetTickCount64");
+
 	/* set the thread execution level to avoid sleep */
 	/* first try for Windows 7 */
 	if (SetThreadExecutionState(WIN32_ES_CONTINUOUS | WIN32_ES_SYSTEM_REQUIRED | WIN32_ES_AWAYMODE_REQUIRED) == 0) {
@@ -154,6 +175,8 @@ void os_done(void)
 	/* delete the thread local storage for strerror() */
 	pthread_key_delete(last_error);
 
+	pthread_mutex_destroy(&tick_lock);
+
 	/* restore the normal execution level */
 	SetThreadExecutionState(WIN32_ES_CONTINUOUS);
 
@@ -478,6 +501,8 @@ static int windows_info2stat(const BY_HANDLE_FILE_INFORMATION* info, const FILE_
 	st->st_ino <<= 32;
 	st->st_ino |= info->nFileIndexLow;
 
+	st->st_nlink = info->nNumberOfLinks;
+
 	st->st_dev = info->dwVolumeSerialNumber;
 
 	/* GetFileInformationByHandle() ensures to return synced information */
@@ -530,6 +555,8 @@ static int windows_stream2stat(const BY_HANDLE_FILE_INFORMATION* info, const FIL
 
 	st->st_ino = stream->FileId.QuadPart;
 
+	st->st_nlink = info->nNumberOfLinks;
+
 	st->st_dev = info->dwVolumeSerialNumber;
 
 	/* directory listing doesn't ensure to return synced information */
@@ -575,6 +602,9 @@ static void windows_finddata2stat(const WIN32_FIND_DATAW* info, struct windows_s
 	/* No inode information available */
 	st->st_ino = 0;
 
+	/* No link information available */
+	st->st_nlink = 0;
+
 	/* No device information available */
 	st->st_dev = 0;
 
@@ -660,9 +690,12 @@ static void windows_errno(DWORD error)
 	case ERROR_NOT_SUPPORTED : /* in CreateSymlinkW() if not present in kernel32 */
 		errno = ENOSYS;
 		break;
-	case ERROR_PRIVILEGE_NOT_HELD : /* in CreateSymlinkW if no SeCreateSymbolicLinkPrivilige permission */
+	case ERROR_PRIVILEGE_NOT_HELD : /* in CreateSymlinkW() if no SeCreateSymbolicLinkPrivilige permission */
 		errno = EPERM;
 		break;
+	case ERROR_IO_DEVICE : /* in ReadFile() and WriteFile() */
+		errno = EIO;
+		break;
 	default :
 		log_fatal("Unexpected Windows error %d.\n", (int)error);
 		errno = EIO;
@@ -1040,6 +1073,70 @@ int windows_futimens(int fd, struct windows_timespec tv[2])
 	return 0;
 }
 
+int windows_utimensat(int fd, const char* file, struct windows_timespec tv[2], int flags)
+{
+	wchar_t conv_buf[CONV_MAX];
+	HANDLE h;
+	FILETIME ft;
+	uint64_t mtime;
+	DWORD wflags;
+
+	/*
+	 * Support only the absolute paths
+	 */
+	if (fd != AT_FDCWD) {
+		errno = EBADF;
+		return -1;
+	}
+
+	/*
+	 * Open the handle of the file.
+	 *
+	 * Use FILE_FLAG_BACKUP_SEMANTICS to open directories (it's just ignored for files).
+	 * Use FILE_FLAG_OPEN_REPARSE_POINT to open symbolic links and not the their target.
+	 *
+	 * Note that even with FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
+	 * and FILE_FLAG_BACKUP_SEMANTICS | FILE_FLAG_OPEN_REPARSE_POINT some paths
+	 * cannot be opened like "C:\System Volume Information" resulting
+	 * in error ERROR_ACCESS_DENIED.
+	 */
+	wflags = FILE_FLAG_BACKUP_SEMANTICS;
+	if ((flags & AT_SYMLINK_NOFOLLOW) != 0)
+		wflags |= FILE_FLAG_OPEN_REPARSE_POINT;
+	h = CreateFileW(convert(conv_buf, file), FILE_WRITE_ATTRIBUTES, FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE, 0, OPEN_EXISTING, wflags, 0);
+	if (h == INVALID_HANDLE_VALUE) {
+		windows_errno(GetLastError());
+		return -1;
+	}
+
+	/*
+	 * Convert to windows time
+	 *
+	 * How To Convert a UNIX time_t to a Win32 FILETIME or SYSTEMTIME
+	 * http://support.microsoft.com/kb/167296
+	 */
+	mtime = tv[0].tv_sec;
+	mtime *= 10000000;
+	mtime += tv[0].tv_nsec / 100;
+	mtime += 116444736000000000;
+
+	ft.dwHighDateTime = mtime >> 32;
+	ft.dwLowDateTime = mtime;
+
+	if (!SetFileTime(h, 0, 0, &ft)) {
+		windows_errno(GetLastError());
+		CloseHandle(h);
+		return -1;
+	}
+
+	if (!CloseHandle(h)) {
+		windows_errno(GetLastError());
+		return -1;
+	}
+
+	return 0;
+}
+
 int windows_rename(const char* from, const char* to)
 {
 	wchar_t conv_buf_from[CONV_MAX];
@@ -1457,6 +1554,17 @@ int windows_link(const char* existing, const char* file)
 	return 0;
 }
 
+/**
+ * In Windows 10 allow creationg of symblink by not priviliged user.
+ *
+ * See: Symlinks in Windows 10!
+ * https://blogs.windows.com/buildingapps/2016/12/02/symlinks-windows-10/#cQG7cx48oGH86lkI.97
+ * "Specify this flag to allow creation of symbolic links when the process is not elevated"
+ */
+#ifndef SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE
+#define SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE 0x2
+#endif
+
 int windows_symlink(const char* existing, const char* file)
 {
 	wchar_t conv_buf_file[CONV_MAX];
@@ -1466,9 +1574,20 @@ int windows_symlink(const char* existing, const char* file)
 	/* otherwise the link creation fails. */
 	/* But we don't want to always convert it, to avoid to recreate */
 	/* user symlinks different than they were before */
-	if (!CreateSymbolicLinkW(convert(conv_buf_file, file), convert_if_required(conv_buf_existing, existing), 0)) {
-		windows_errno(GetLastError());
-		return -1;
+	if (!CreateSymbolicLinkW(convert(conv_buf_file, file), convert_if_required(conv_buf_existing, existing),
+		SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE)
+	) {
+		DWORD error = GetLastError();
+		if (GetLastError() != ERROR_INVALID_PARAMETER) {
+			windows_errno(error);
+			return -1;
+		}
+
+		/* retry without the new flag SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE */
+		if (!CreateSymbolicLinkW(convert(conv_buf_file, file), convert_if_required(conv_buf_existing, existing), 0)) {
+			windows_errno(GetLastError());
+			return -1;
+		}
 	}
 
 	return 0;
@@ -1596,7 +1715,7 @@ int filephy(const char* file, uint64_t size, uint64_t* physical)
 	return 0;
 }
 
-int fsinfo(const char* path, int* has_persistent_inode, uint64_t* total_space, uint64_t* free_space)
+int fsinfo(const char* path, int* has_persistent_inode, int* has_syncronized_hardlinks, uint64_t* total_space, uint64_t* free_space)
 {
 	wchar_t conv_buf[CONV_MAX];
 
@@ -1604,6 +1723,10 @@ int fsinfo(const char* path, int* has_persistent_inode, uint64_t* total_space, u
 	if (has_persistent_inode)
 		*has_persistent_inode = 1;
 
+	/* NTFS doesn't syncronize hardlinks metadata */
+	if (has_syncronized_hardlinks)
+		*has_syncronized_hardlinks = 0;
+
 	if (free_space || total_space) {
 		ULARGE_INTEGER total_bytes;
 		ULARGE_INTEGER total_free_bytes;
@@ -1892,16 +2015,40 @@ size_t windows_direct_size(void)
 uint64_t tick(void)
 {
 	LARGE_INTEGER t;
+	uint64_t r;
 
-	if (!QueryPerformanceCounter(&t))
-		return 0;
+	/*
+	 * Ensure to return a strict monotone tick counter.
+	 *
+	 * We had reports of invalid stats due faulty High Precision Event Timer.
+	 * See: https://sourceforge.net/p/snapraid/discussion/1677233/thread/a2122fd6/
+	 */
+	pthread_mutex_lock(&tick_lock);
 
-	return t.QuadPart;
+	/*
+	 * MSDN 'QueryPerformanceCounter'
+	 * "On systems that run Windows XP or later, the function"
+	 * "will always succeed and will thus never return zero."
+	 */
+	r = 0;
+	if (QueryPerformanceCounter(&t))
+		r = t.QuadPart;
+
+	if (r < tick_last)
+		r = tick_last;
+	tick_last = r;
+
+	pthread_mutex_unlock(&tick_lock);
+
+	return r;
 }
 
 uint64_t tick_ms(void)
 {
 	/* GetTickCount64() isn't supported in Windows XP */
+	if (ptr_GetTickCount64 != 0)
+		return ptr_GetTickCount64();
+
 	return GetTickCount();
 }
 
