@@ -26,7 +26,7 @@
 /**
  * Pseudo random limits for parity
  */
-#define PARITY_LIMIT(size,split,level) \
+#define PARITY_LIMIT(size, split, level) \
 	size ? size + (123562341 + split * 634542351 + level * 983491341) % size : 0
 
 /****************************************************************************/
@@ -165,7 +165,7 @@ void parity_size(struct snapraid_parity_handle* handle, data_off_t* out_size)
 	/* now compute the size summing all the parity splits */
 	size = 0;
 
-	for (s = 0;s < handle->split_mac;++s) {
+	for (s = 0; s < handle->split_mac; ++s) {
 		struct snapraid_split_handle* split = &handle->split_map[s];
 
 		size += split->size;
@@ -185,7 +185,7 @@ int parity_create(struct snapraid_parity_handle* handle, const struct snapraid_p
 	handle->level = level;
 	handle->split_mac = 0;
 
-	for (s = 0;s < parity->split_mac;++s) {
+	for (s = 0; s < parity->split_mac; ++s) {
 		struct snapraid_split_handle* split = &handle->split_map[s];
 		int ret;
 		int flags;
@@ -247,7 +247,7 @@ int parity_create(struct snapraid_parity_handle* handle, const struct snapraid_p
 
 bail:
 	/* LCOV_EXCL_START */
-	for (s = 0;s < handle->split_mac;++s) {
+	for (s = 0; s < handle->split_mac; ++s) {
 		struct snapraid_split_handle* split = &handle->split_map[s];
 		close(split->f);
 		split->f = -1;
@@ -256,10 +256,12 @@ bail:
 	/* LCOV_EXCL_STOP */
 }
 
-static int parity_handle_grow(struct snapraid_split_handle* split, data_off_t size, int skip_fallocate)
+static int parity_handle_grow(struct snapraid_split_handle* split, data_off_t previous_size, data_off_t size, int skip_fallocate)
 {
 	int ret;
 
+	(void)previous_size;
+
 	/* simulate a failure for testing limits */
 	if (split->limit_size != 0 && size > (data_off_t)split->limit_size)
 		return -1;
@@ -276,8 +278,14 @@ static int parity_handle_grow(struct snapraid_split_handle* split, data_off_t si
 		 *
 		 * See: fallocate vs posix_fallocate
 		 * http://stackoverflow.com/questions/14063046/fallocate-vs-posix-fallocate
+		 *
+		 * To work better with Btrfs, use as offset the previous allocated size.
+		 * Otherwise Btrfs will count as space needed even the already allocated one.
+		 *
+		 * See: Massive loss of disk space
+		 * https://www.mail-archive.com/linux-btrfs@vger.kernel.org/msg66454.html
 		 */
-		ret = fallocate(split->f, 0, 0, size);
+		ret = fallocate(split->f, 0, previous_size, size - previous_size);
 
 		/*
 		 * In some legacy system fallocate() may return the error number
@@ -311,7 +319,7 @@ static int parity_handle_grow(struct snapraid_split_handle* split, data_off_t si
 	 *
 	 * See: man fallocate
 	 * ENOSYS - This kernel does not implement fallocate().
-         * EOPNOTSUPP - The file system containing the file referred to by fd does not support this operation
+	 * EOPNOTSUPP - The file system containing the file referred to by fd does not support this operation
 	 */
 	if (ret != 0 && (errno == EOPNOTSUPP || errno == ENOSYS)) {
 		/* fallback using ftruncate() */
@@ -324,12 +332,26 @@ static int parity_handle_grow(struct snapraid_split_handle* split, data_off_t si
 	ret = ftruncate(split->f, size);
 #endif
 
+	if (ret != 0)
+		log_tag("split:grow:%s:%" PRIu64 ": failed with error %s\n", split->path, size, strerror(errno));
+	else
+		log_tag("split:grow:%s:%" PRIu64 ": ok\n", split->path, size);
+
 	return ret;
 }
 
 static int parity_handle_shrink(struct snapraid_split_handle* split, data_off_t size)
 {
-	return ftruncate(split->f, size);
+	int ret;
+
+	ret = ftruncate(split->f, size);
+
+	if (ret != 0)
+		log_tag("split:shrink:%s:%" PRIu64 ": failed with error %s\n", split->path, size, strerror(errno));
+	else
+		log_tag("split:shrink:%s:%" PRIu64 ": ok\n", split->path, size);
+
+	return ret;
 }
 
 /**
@@ -414,6 +436,8 @@ static int parity_handle_fill(struct snapraid_split_handle* split, data_off_t si
 	/* size we have to increase */
 	delta = size - base;
 
+	log_tag("split:fill:%s:%" PRIu64 ":%" PRIu64 ":\n", split->path, base, size);
+
 	/* grow the size one bit at time, like a kind of binary search */
 	while (delta != 0) {
 		int ret;
@@ -422,7 +446,9 @@ static int parity_handle_fill(struct snapraid_split_handle* split, data_off_t si
 		/* mask out the bit we process */
 		delta &= ~run;
 
-		ret = parity_handle_grow(split, base + run, skip_fallocate);
+		log_tag("split:delta:%s:%" PRIu64 ":%" PRIu64 ":\n", split->path, base, run);
+
+		ret = parity_handle_grow(split, base, base + run, skip_fallocate);
 		if (ret != 0) {
 			/* we cannot grow, fallback enabling all the smaller bits */
 			delta = run - 1;
@@ -536,7 +562,7 @@ int parity_chsize(struct snapraid_parity_handle* handle, struct snapraid_parity*
 		/* LCOV_EXCL_STOP */
 	}
 
-	for (s = 0;s < handle->split_mac;++s) {
+	for (s = 0; s < handle->split_mac; ++s) {
 		struct snapraid_split_handle* split = &handle->split_map[s];
 		int is_fixed = parity_split_is_fixed(handle, s);
 		data_off_t run;
@@ -564,7 +590,7 @@ int parity_chsize(struct snapraid_parity_handle* handle, struct snapraid_parity*
 			/* otherwise tries to allocate all the needed remaining size */
 			run = size;
 		}
-		
+
 		ret = parity_handle_chsize(split, run, block_size, skip_fallocate, skip_space_holder);
 		if (ret != 0) {
 			/* LCOV_EXCL_START */
@@ -574,12 +600,12 @@ int parity_chsize(struct snapraid_parity_handle* handle, struct snapraid_parity*
 
 		if (split->st.st_size > run) {
 			/* LCOV_EXCL_START */
-			log_fatal("Unexpected over resizing parity file '%s' to size %" PRIu64 " resulting in size %" PRIu64 ".\n", split->path, run, split->st.st_size);
+			log_fatal("Unexpected over resizing parity file '%s' to size %" PRIu64 " resulting in size %" PRIu64 ".\n", split->path, run, (uint64_t)split->st.st_size);
 			return -1;
 			/* LCOV_EXCL_STOP */
 		} else if (is_fixed && split->st.st_size < run) {
 			/* LCOV_EXCL_START */
-			log_fatal("Failed restoring parity file '%s' to size %" PRIu64 " resulting in size %" PRIu64 ".\n", split->path, run, split->st.st_size);
+			log_fatal("Failed restoring parity file '%s' to size %" PRIu64 " resulting in size %" PRIu64 ".\n", split->path, run, (uint64_t)split->st.st_size);
 			return -1;
 			/* LCOV_EXCL_STOP */
 		} else {
@@ -613,7 +639,7 @@ int parity_chsize(struct snapraid_parity_handle* handle, struct snapraid_parity*
 	if (is_modified)
 		*is_modified = 0;
 
-	for (s = 0;s < handle->split_mac;++s) {
+	for (s = 0; s < handle->split_mac; ++s) {
 		struct snapraid_split_handle* split = &handle->split_map[s];
 
 		if (parity->split_map[s].size != split->size) {
@@ -637,7 +663,7 @@ int parity_open(struct snapraid_parity_handle* handle, const struct snapraid_par
 	/* mask of bits used by the block size */
 	block_mask = ((data_off_t)block_size) - 1;
 
-	for (s = 0;s < parity->split_mac;++s) {
+	for (s = 0; s < parity->split_mac; ++s) {
 		struct snapraid_split_handle* split = &handle->split_map[s];
 		int ret;
 		int flags;
@@ -701,7 +727,7 @@ int parity_open(struct snapraid_parity_handle* handle, const struct snapraid_par
 
 bail:
 	/* LCOV_EXCL_START */
-	for (s = 0;s < handle->split_mac;++s) {
+	for (s = 0; s < handle->split_mac; ++s) {
 		struct snapraid_split_handle* split = &handle->split_map[s];
 		close(split->f);
 		split->f = -1;
@@ -715,7 +741,7 @@ int parity_sync(struct snapraid_parity_handle* handle)
 #if HAVE_FSYNC
 	unsigned s;
 
-	for (s = 0;s < handle->split_mac;++s) {
+	for (s = 0; s < handle->split_mac; ++s) {
 		struct snapraid_split_handle* split = &handle->split_map[s];
 		int ret;
 
@@ -740,7 +766,7 @@ int parity_close(struct snapraid_parity_handle* handle)
 	unsigned s;
 	int f_ret = 0;
 
-	for (s = 0;s < handle->split_mac;++s) {
+	for (s = 0; s < handle->split_mac; ++s) {
 		struct snapraid_split_handle* split = &handle->split_map[s];
 		int ret;
 
@@ -771,7 +797,7 @@ struct snapraid_split_handle* parity_split_find(struct snapraid_parity_handle* h
 	if (*offset < 0)
 		return 0;
 
-	for (s = 0;s < handle->split_mac;++s) {
+	for (s = 0; s < handle->split_mac; ++s) {
 		struct snapraid_split_handle* split = &handle->split_map[s];
 
 		if (*offset < split->size)
