@@ -501,11 +501,16 @@ static void scan_file_refresh(struct snapraid_scan* scan, const char* sub, struc
 			 * because the metadata in the directory is updated only when the file
 			 * is closed.
 			 *
+			 * The same happens for hardlinks that duplicate metatada.
+			 * The link metadata is updated only when the link is opened.
+			 * This extends also to st_size and st_nlink.
+			 *
 			 * See also:
 			 * Why is the file size reported incorrectly for files that are still being written to?
 			 * http://blogs.msdn.com/b/oldnewthing/archive/2011/12/26/10251026.aspx
 			 */
-			log_fatal("WARNING! Detected uncached time change for file '%s'\n", sub);
+			log_fatal("WARNING! Detected uncached time change from %" PRIu64 ".%09u to %" PRIu64 ".%09u for file '%s'\n",
+				(uint64_t)st->st_mtime, (uint32_t)st->st_mtimensec, (uint64_t)synced_st.st_mtime, (uint32_t)synced_st.st_mtimensec, sub);
 			log_fatal("It's better if you run SnapRAID without other processes running.\n");
 #endif
 			st->st_mtime = synced_st.st_mtime;
@@ -514,15 +519,26 @@ static void scan_file_refresh(struct snapraid_scan* scan, const char* sub, struc
 
 		if (st->st_size != synced_st.st_size) {
 #ifndef _WIN32
-			log_fatal("WARNING! Detected uncached size change for file '%s'\n", sub);
+			log_fatal("WARNING! Detected uncached size change from %" PRIu64 " to %" PRIu64 " for file '%s'\n",
+				(uint64_t)st->st_size, (uint64_t)synced_st.st_size, sub);
 			log_fatal("It's better if you run SnapRAID without other processes running.\n");
 #endif
 			st->st_size = synced_st.st_size;
 		}
 
+		if (st->st_nlink != synced_st.st_nlink) {
+#ifndef _WIN32
+			log_fatal("WARNING! Detected uncached nlink change from %u to %u for file '%s'\n",
+				(uint32_t)st->st_nlink, (uint32_t)synced_st.st_nlink, sub);
+			log_fatal("It's better if you run SnapRAID without other processes running.\n");
+#endif
+			st->st_nlink = synced_st.st_nlink;
+		}
+
 		if (st->st_ino != synced_st.st_ino) {
-			log_fatal("DANGER! Detected uncached inode change for file '%s'\n", sub);
-			log_fatal("Please ensure to run SnapRAID without other processes running.\n");
+			log_fatal("DANGER! Detected uncached inode change from %" PRIu64 " to %" PRIu64 " for file '%s'\n",
+				(uint64_t)st->st_ino, (uint64_t)synced_st.st_ino, sub);
+			log_fatal("It's better if you run SnapRAID without other processes running.\n");
 			/* at this point, it's too late to change inode */
 			/* and having inconsistent inodes may result to internal failures */
 			/* so, it's better to abort */
@@ -666,14 +682,14 @@ static void scan_file(struct snapraid_scan* scan, int is_diff, const char* sub,
 		) {
 			/* check if multiple files have the same inode */
 			if (file_flag_has(file, FILE_IS_PRESENT)) {
-#if HAVE_STRUCT_STAT_ST_NLINK
-				if (st->st_nlink <= 1) {
+				/* if has_volatile_hardlinks is true, the nlink value is not reliable */
+				if (!disk->has_volatile_hardlinks && st->st_nlink == 1) {
 					/* LCOV_EXCL_START */
 					log_fatal("Internal inode '%" PRIu64 "' inconsistency for file '%s%s' already present\n", (uint64_t)st->st_ino, disk->dir, sub);
 					os_abort();
 					/* LCOV_EXCL_STOP */
 				}
-#endif
+
 				/* it's a hardlink */
 				scan_link(scan, is_diff, sub, file->sub, FILE_IS_HARDLINK);
 				return;
@@ -729,16 +745,42 @@ static void scan_file(struct snapraid_scan* scan, int is_diff, const char* sub,
 			return;
 		}
 
-		/* here the file matches the inode, but not the other info */
-		/* if could be a modified file with the same name, */
-		/* or a restored/copied file that get assigned a previously used inode, */
-		/* or a file-system with not persistent inodes */
-
-		/* for sure it cannot be already present */
+		/*
+		 * Here the file matches the inode, but not the other info
+		 *
+		 * It could be a modified file with the same name,
+		 * or a restored/copied file that get assigned a previously used inode,
+		 * or a file-system with not persistent inodes.
+		 *
+		 * In NTFS it could be also a hardlink, because in NTFS
+		 * hardlink don't share the same directory information,
+		 * like attribute and time.
+		 *
+		 * For example:
+		 *   C:> echo A > A
+		 *   C:> mklink /H B A
+		 *   ...wait one minute
+		 *   C:> echo AAAAAAAAAAAAAA > A
+		 *   C:> dir
+		 *   ...both time and size of A and B don't match!
+		 */
 		if (file_flag_has(file, FILE_IS_PRESENT)) {
+			/* if has_volatile_hardlinks is true, the nlink value is not reliable */
+			if (!disk->has_volatile_hardlinks && st->st_nlink == 1) {
+				/* LCOV_EXCL_START */
+				log_fatal("Internal inode '%" PRIu64 "' inconsistency for files '%s%s' and '%s%s' with same inode but different attributes: size %" PRIu64 "?%" PRIu64 ", sec %" PRIu64 "?%" PRIu64 ", nsec %d?%d\n",
+					file->inode, disk->dir, sub, disk->dir, file->sub,
+					file->size, (uint64_t)st->st_size,
+					file->mtime_sec, (uint64_t)st->st_mtime,
+					file->mtime_nsec, STAT_NSEC(st));
+				os_abort();
+				/* LCOV_EXCL_STOP */
+			}
+
 			/* LCOV_EXCL_START */
-			log_fatal("Internal inode '%" PRIu64 "' inconsistency for files '%s%s' and '%s%s' matching and already present but different\n", file->inode, disk->dir, sub, disk->dir, file->sub);
-			os_abort();
+			/* suppose it's hardlink with not synced metadata */
+			scan_link(scan, is_diff, sub, file->sub, FILE_IS_HARDLINK);
+			return;
 			/* LCOV_EXCL_STOP */
 		}
 
@@ -970,8 +1012,8 @@ static void scan_file(struct snapraid_scan* scan, int is_diff, const char* sub,
 			++scan->count_change;
 
 			log_tag("scan:update:%s:%s: %" PRIu64 " %" PRIu64 ".%d -> %" PRIu64 " %" PRIu64 ".%d\n", disk->name, esc_tag(sub, esc_buffer),
-				(uint64_t)file_already_present_size, (uint64_t)file_already_present_mtime_sec, file_already_present_mtime_nsec,
-				(uint64_t)file->size, (uint64_t)file->mtime_sec, file->mtime_nsec
+				file_already_present_size, file_already_present_mtime_sec, file_already_present_mtime_nsec,
+				file->size, file->mtime_sec, file->mtime_nsec
 			);
 
 			if (is_diff) {
@@ -1328,7 +1370,7 @@ static int scan_dir(struct snapraid_scan* scan, int level, int is_diff, const ch
 #if HAVE_LSTAT_SYNC
 				/* if the st_ino field is missing, takes care to fill it using the extended lstat() */
 				/* this can happen only in Windows */
-				if (st->st_ino == 0) {
+				if (st->st_ino == 0 || st->st_nlink == 0) {
 					if (lstat_sync(path_next, st, 0) != 0) {
 						/* LCOV_EXCL_START */
 						log_fatal("Error in stat file '%s'. %s.\n", path_next, strerror(errno));
@@ -1447,7 +1489,8 @@ static int state_diffscan(struct snapraid_state* state, int is_diff)
 		struct snapraid_scan* scan;
 		tommy_node* node;
 		int ret;
-		int has_persistent_inode;
+		int has_persistent_inodes;
+		int has_syncronized_hardlinks;
 
 		scan = malloc_nofail(sizeof(struct snapraid_scan));
 		scan->state = state;
@@ -1469,16 +1512,19 @@ static int state_diffscan(struct snapraid_state* state, int is_diff)
 			msg_progress("Scanning disk %s...\n", disk->name);
 
 		/* check if the disk supports persistent inodes */
-		ret = fsinfo(disk->dir, &has_persistent_inode, 0, 0);
+		ret = fsinfo(disk->dir, &has_persistent_inodes, &has_syncronized_hardlinks, 0, 0);
 		if (ret < 0) {
 			/* LCOV_EXCL_START */
 			log_fatal("Error accessing disk '%s' to get file-system info. %s.\n", disk->dir, strerror(errno));
 			exit(EXIT_FAILURE);
 			/* LCOV_EXCL_STOP */
 		}
-		if (!has_persistent_inode) {
+		if (!has_persistent_inodes) {
 			disk->has_volatile_inodes = 1;
 		}
+		if (!has_syncronized_hardlinks) {
+			disk->has_volatile_hardlinks = 1;
+		}
 
 		/* if inodes or UUID are not persistent/changed/unsupported */
 		if (disk->has_volatile_inodes || disk->has_different_uuid || disk->has_unsupported_uuid) {
