@@ -527,32 +527,29 @@ const char* esc_shell_multi(const char** str_map, unsigned str_max, char* buffer
 				*p++ = c;
 			}
 			break;
-		default :
-			/* unquoted */
-			if (p == end)
-				goto bail;
-			*p++ = c;
-			break;
 #else
-		case 'a' ... 'z' :
-		case 'A' ... 'Z' :
-		case '0' ... '9' :
-		case ',' :
-		case '.' :
-		case '_' :
-		case '+' :
-		case ':' :
-		case '@' :
-		case '%' :
-		case '/' :
-		case '-' :
-			/* safe chars for all shells, unquoted */
-			if (p == end)
-				goto bail;
-			*p++ = c;
-			break;
-		default :
-			/* all the others are quoted with \ */
+		/* special chars that need to be quoted */
+		case ' ' : /* space */
+		case '~' : /* home */
+		case '`' : /* command */
+		case '#' : /* comment */
+		case '$' : /* variable */
+		case '&' : /* background job */
+		case '*' : /* wildcard */
+		case '(' : /* shell */
+		case ')' : /* shell */
+		case '\\': /* quote */
+		case '|' : /* pipe */
+		case '[' : /* wildcard */
+		case ']' : /* wildcard */
+		case '{' : /* code */
+		case '}' : /* code */
+		case ';' : /* separator */
+		case '\'': /* quote */
+		case '"' : /* quote */
+		case '<' : /* redirect */
+		case '>' : /* redirect */
+		case '?' : /* wildcard */
 			if (p == end)
 				goto bail;
 			*p++ = '\\';
@@ -561,6 +558,12 @@ const char* esc_shell_multi(const char** str_map, unsigned str_max, char* buffer
 			*p++ = c;
 			break;
 #endif
+		default :
+			/* unquoted */
+			if (p == end)
+				goto bail;
+			*p++ = c;
+			break;
 		}
 
 		++str;
@@ -894,6 +897,52 @@ int fmtime(int f, int64_t mtime_sec, int mtime_nsec)
 	return ret;
 }
 
+int lmtime(const char* path, int64_t mtime_sec, int mtime_nsec)
+{
+#if HAVE_UTIMENSAT
+	struct timespec tv[2];
+#else
+	struct timeval tv[2];
+#endif
+	int ret;
+
+#if HAVE_UTIMENSAT /* utimensat() is preferred because it gives nanosecond precision */
+	tv[0].tv_sec = mtime_sec;
+	if (mtime_nsec != STAT_NSEC_INVALID)
+		tv[0].tv_nsec = mtime_nsec;
+	else
+		tv[0].tv_nsec = 0;
+	tv[1].tv_sec = tv[0].tv_sec;
+	tv[1].tv_nsec = tv[0].tv_nsec;
+
+	ret = utimensat(AT_FDCWD, path, tv, AT_SYMLINK_NOFOLLOW);
+#elif HAVE_LUTIMES /* fallback to lutimes() if nanosecond precision is not available */
+	tv[0].tv_sec = mtime_sec;
+	if (mtime_nsec != STAT_NSEC_INVALID)
+		tv[0].tv_usec = mtime_nsec / 1000;
+	else
+		tv[0].tv_usec = 0;
+	tv[1].tv_sec = tv[0].tv_sec;
+	tv[1].tv_usec = tv[0].tv_usec;
+
+	ret = lutimes(path, tv);
+#elif HAVE_FUTIMESAT /* fallback to futimesat() for Solaris, it only has futimesat() */
+	tv[0].tv_sec = mtime_sec;
+	if (mtime_nsec != STAT_NSEC_INVALID)
+		tv[0].tv_usec = mtime_nsec / 1000;
+	else
+		tv[0].tv_usec = 0;
+	tv[1].tv_sec = tv[0].tv_sec;
+	tv[1].tv_usec = tv[0].tv_usec;
+
+	ret = futimesat(AT_FDCWD, path, tv);
+#else
+#error No function available to set file timestamps with sub-second precision
+#endif
+
+	return ret;
+}
+
 /****************************************************************************/
 /* advise */
 
