@@ -114,6 +114,7 @@ static int devresolve_proc(uint64_t device, char* path, size_t path_size)
 		char* separator;
 		char* majorminor;
 		char* mountpoint;
+		char* fs;
 		char* mountsource;
 
 		s = fgets(buf, sizeof(buf), f);
@@ -141,18 +142,31 @@ static int devresolve_proc(uint64_t device, char* path, size_t path_size)
 
 		majorminor = first_map[2];
 		mountpoint = first_map[4];
+		fs = second_map[0];
 		mountsource = second_map[1];
 
-		/* accept only /dev/... mountsource */
-		/* this excludes ZFS that has the bare label, like "tank" */
-		if (strncmp(mountsource, "/dev/", 5) != 0)
-			continue;
-
 		/* compare major:minor from mountinfo */
 		if (strcmp(majorminor, match) == 0) {
+			/*
+			 * Accept only /dev/... mountsource
+			 *
+			 * This excludes ZFS that uses a bare label for mountsource, like "tank".
+			 *
+			 * 410 408 0:193 / /XXX rw,relatime shared:217 - zfs tank/system/data/var/lib/docker/XXX rw,xattr,noacl
+			 *
+			 * Also excludes AUTOFS unmounted devices that point to a fake filesystem
+			 * used to remount them at the first use.
+			 *
+			 * 97 25 0:42 / /XXX rw,relatime shared:76 - autofs /etc/auto.seed rw,fd=6,pgrp=952,timeout=30,minproto=5,maxproto=5,indirect
+			 */
+			if (strncmp(mountsource, "/dev/", 5) != 0) {
+				log_tag("resolve:proc:%u:%u: match skipped for not /dev/ mountsource for %s %s\n", major(device), minor(device), fs, mountsource);
+				continue;
+			}
+
 			pathcpy(path, path_size, mountsource);
 
-			log_tag("resolve:proc:%u:%u:%s: found mountinfo device %s\n", major(device), minor(device), path, match);
+			log_tag("resolve:proc:%u:%u: found device %s matching device %s\n", major(device), minor(device), path, match);
 
 			fclose(f);
 			return 0;
@@ -161,9 +175,14 @@ static int devresolve_proc(uint64_t device, char* path, size_t path_size)
 		/* get the device of the mount point */
 		/* in Btrfs it could be different than the one in mountinfo */
 		if (stat(mountpoint, &st) == 0 && st.st_dev == device) {
+			if (strncmp(mountsource, "/dev/", 5) != 0) {
+				log_tag("resolve:proc:%u:%u: match skipped for not /dev/ mountsource for %s %s\n", major(device), minor(device), fs, mountsource);
+				continue;
+			}
+
 			pathcpy(path, path_size, mountsource);
 
-			log_tag("resolve:proc:%u:%u:%s: found mountpoint %s\n", major(device), minor(device), path, mountpoint);
+			log_tag("resolve:proc:%u:%u: found device %s matching mountpoint %s\n", major(device), minor(device), path, mountpoint);
 
 			fclose(f);
 			return 0;
@@ -404,7 +423,7 @@ static int devuuid_dev(uint64_t device, char* uuid, size_t uuid_size)
 	/* scan the UUID directory searching for the device */
 	d = opendir("/dev/disk/by-uuid");
 	if (!d) {
-		log_tag("uuid:by-uuidd:%u:%u: opendir(/dev/disk/by-uuid) failed, %s\n", major(device), minor(device), strerror(errno));
+		log_tag("uuid:by-uuid:%u:%u: opendir(/dev/disk/by-uuid) failed, %s\n", major(device), minor(device), strerror(errno));
 		/* directory missing?, likely we are not in Linux */
 		return -1;
 	}
@@ -416,7 +435,7 @@ static int devuuid_dev(uint64_t device, char* uuid, size_t uuid_size)
 
 		ret = fstatat(dirfd(d), dd->d_name, &st, 0);
 		if (ret != 0) {
-			log_tag("uuid:by-uuidd:%u:%u: fstatat(%s) failed, %s\n", major(device), minor(device), dd->d_name, strerror(errno));
+			log_tag("uuid:by-uuid:%u:%u: fstatat(%s) failed, %s\n", major(device), minor(device), dd->d_name, strerror(errno));
 			/* generic error, ignore and continue the search */
 			continue;
 		}
@@ -429,8 +448,8 @@ static int devuuid_dev(uint64_t device, char* uuid, size_t uuid_size)
 			/* resolve the link */
 			pathprint(path, sizeof(path), "/dev/disk/by-uuid/%s", dd->d_name);
 			ret = readlink(path, buf, sizeof(buf));
-			if (ret < 0 || ret == sizeof(buf)) {
-				log_tag("uuid:by-uuidd:%u:%u: readlink(/dev/disk/by-uuid/%s) failed, %s\n", major(device), minor(device), dd->d_name, strerror(errno));
+			if (ret < 0 || ret >= PATH_MAX) {
+				log_tag("uuid:by-uuid:%u:%u: readlink(/dev/disk/by-uuid/%s) failed, %s\n", major(device), minor(device), dd->d_name, strerror(errno));
 				/* generic error, ignore and continue the search */
 				continue;
 			}
@@ -446,7 +465,7 @@ static int devuuid_dev(uint64_t device, char* uuid, size_t uuid_size)
 		}
 	}
 
-	log_tag("uuid:by-uuidd:%u:%u: /dev/disk/by-uuid doesn't contain a matching block device\n", major(device), minor(device));
+	log_tag("uuid:by-uuid:%u:%u: /dev/disk/by-uuid doesn't contain a matching block device\n", major(device), minor(device));
 
 	/* not found */
 	closedir(d);
@@ -512,7 +531,7 @@ int devuuid(uint64_t device, char* uuid, size_t uuid_size)
 	if (devuuid_dev(device, uuid, uuid_size) == 0)
 		return 0;
 #else
-	log_tag("uuid:by-uuidd:%u:%u: by-uuid not supported\n", major(device), minor(device));
+	log_tag("uuid:by-uuid:%u:%u: by-uuid not supported\n", major(device), minor(device));
 #endif
 
 	/* fall back to blkid for other cases */
@@ -520,7 +539,7 @@ int devuuid(uint64_t device, char* uuid, size_t uuid_size)
 	if (devuuid_blkid(device, uuid, uuid_size) == 0)
 		return 0;
 #else
-	log_tag("uuid:blkid:%u:%u: blkid not supported\n", major(device), minor(device));
+	log_tag("uuid:blkid:%u:%u: blkid support not compiled in\n", major(device), minor(device));
 #endif
 
 	log_tag("uuid:notfound:%u:%u:\n", major(device), minor(device));
@@ -625,7 +644,7 @@ int filephy(const char* path, uint64_t size, uint64_t* physical)
 	return 0;
 }
 
-int fsinfo(const char* path, int* has_persistent_inode, uint64_t* total_space, uint64_t* free_space)
+int fsinfo(const char* path, int* has_persistent_inode, int* has_syncronized_hardlinks, uint64_t* total_space, uint64_t* free_space)
 {
 	char type[64];
 	const char* ptype;
@@ -674,11 +693,29 @@ int fsinfo(const char* path, int* has_persistent_inode, uint64_t* total_space, u
 			break;
 		}
 #else
-		/* by default assume yes */
+		/* in Unix inodes are persistent by default */
 		*has_persistent_inode = 1;
 #endif
 	}
 
+	if (has_syncronized_hardlinks) {
+#if HAVE_STATFS && HAVE_STRUCT_STATFS_F_TYPE
+		switch (st.f_type) {
+		case 0x5346544E : /* NTFS */
+		case 0x4d44 : /* VFAT, "msdos" in the stat command */
+			*has_syncronized_hardlinks = 0;
+			break;
+		default :
+			/* by default assume yes */
+			*has_syncronized_hardlinks = 1;
+			break;
+		}
+#else
+		/* in Unix hardlinks share the same metadata by default */
+		*has_syncronized_hardlinks = 1;
+#endif
+	}
+
 	if (total_space) {
 #if HAVE_STATFS
 		*total_space = st.f_bsize * (uint64_t)st.f_blocks;
